//
// BioArray Software Environment (BASE) - homepage http://base.thep.lu.se/
// Copyright (C) 2002 Bjrn Samuelsson
//
// This file is part of BASE.
//
// BASE is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// BASE is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
#ifndef BASE_MEDIAN_H
#define BASE_MEDIAN_H

#include <utility>
#include <iterator>
#include <algorithm>

template <class RandomAccessIterator>
std::pair<RandomAccessIterator, RandomAccessIterator>
	median(RandomAccessIterator first, RandomAccessIterator last)
{
	typedef typename
		std::iterator_traits<RandomAccessIterator>::difference_type
		difference_type;

	const difference_type n = std::distance(first, last);
	const RandomAccessIterator mid = first + n/2;

	std::nth_element(first, mid, last);
	if(n % 2)
		return std::pair<RandomAccessIterator, RandomAccessIterator>(mid, mid);
	return std::pair<RandomAccessIterator, RandomAccessIterator>
		(std::max_element(first, mid), mid);
}

template <class RandomAccessIterator, class StrictWeakOrdering>
std::pair<RandomAccessIterator, RandomAccessIterator>
	median(RandomAccessIterator first, RandomAccessIterator last,
	StrictWeakOrdering comp)
{
	typedef typename
		std::iterator_traits<RandomAccessIterator>::difference_type
		difference_type;

	const difference_type n = std::distance(first, last);
	const RandomAccessIterator mid = first + n/2;

	std::nth_element(first, mid, last, comp);
	if(n % 2)
		return std::pair<RandomAccessIterator, RandomAccessIterator>(mid, mid);
	return std::pair<RandomAccessIterator, RandomAccessIterator>
		(std::max_element(first, mid, comp), mid);
}

#endif

